import 'dart:convert';
import 'dart:typed_data';

import 'package:example/save_image.dart';
import 'package:feedback/feedback.dart';
import 'package:http/http.dart' as http;

/// Example usage:
/// ```
/// RaisedButton(
///   child: Text('Click me'),
///   onPressed: (){
///     BetterFeedback.of(context)?.show(uploadToGitLab(
///       'gitlab-project-id',
///       'gitlab-api-token'
///     );
///   }
/// )
/// ```
/// The API token needs access to:
///   - read_api
///   - write_repository
/// See https://docs.gitlab.com/ee/user/project/settings/project_access_tokens.html#limiting-scopes-of-a-project-access-token
OnFeedbackCallback uploadToGitLab(String projectId, String apiToken) {
  return (String feedbackText, Uint8List? feedbackScreenshot) async {
    final screenshotFilePath = await writeImageToStorage(feedbackScreenshot!);

    // Upload screenshot
    final uploadRequest = http.MultipartRequest(
      'POST',
      Uri.https(
        'gitlab.com',
        '/api/v4/projects/$projectId/uploads',
      ),
    )
      ..files.add(await http.MultipartFile.fromPath(
        'file',
        screenshotFilePath,
      ))
      ..headers.putIfAbsent('PRIVATE-TOKEN', () => apiToken);
    final uploadResponse = await uploadRequest.send();

    final dynamic uploadResponseMap = jsonDecode(
      await uploadResponse.stream.bytesToString(),
    );

    // Create issue
    await http.post(
      Uri.https(
        'gitlab.com',
        '/api/v4/projects/$projectId/issues',
        <String, String>{
          'title': feedbackText.padRight(80),
          'description': '$feedbackText\n'
              "${uploadResponseMap["markdown"] ?? "Missing image!"}",
        },
      ),
      headers: {
        'PRIVATE-TOKEN': apiToken,
      },
    );
  };
}
